@@ -8,7 +8,7 @@ package Net::Async::IRC::Protocol;
 use strict;
 use warnings;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use base qw( IO::Async::Stream Protocol::IRC );
 
@@ -38,6 +38,73 @@ parameters, and methods documented below are relevant there.
 
 =cut
 
+=head1 EVENTS
+
+The following events are invoked, either using subclass methods or C<CODE>
+references in parameters:
+
+=head2 $handled = on_message
+
+=head2 $handled = on_message_MESSAGE
+
+Invoked on receipt of a valid IRC message. See C<MESSAGE HANDLING> below.
+
+=head2 on_irc_error $err
+
+Invoked on receipt of an invalid IRC message if parsing fails. C<$err> is the
+error message text. If left unhandled, any parse error will result in the
+connection being immediataely closed, followed by the exception being
+re-thrown.
+
+=head2 on_ping_timeout
+
+Invoked if the peer fails to respond to a C<PING> message within the given
+timeout.
+
+=head2 on_pong_reply $lag
+
+Invoked when the peer successfully sends a C<PONG> reply response to a C<PING>
+message. C<$lag> is the response time in (fractional) seconds.
+
+=cut
+
+=head1 PARAMETERS
+
+The following named parameters may be passed to C<new> or C<configure>:
+
+=over 8
+
+=item on_message => CODE
+
+=item on_message_MESSAGE => CODE
+
+=item on_irc_error => CODE
+
+=item on_ping_timeout => CODE
+
+=item on_pong_reply => CODE
+
+C<CODE> references for event handlers.
+
+=item pingtime => NUM
+
+Amount of quiet time, in seconds, after a message is received from the peer,
+until a C<PING> will be sent to check it is still alive.
+
+=item pongtime => NUM
+
+Timeout, in seconds, after sending a C<PING> message, to wait for a C<PONG>
+response.
+
+=item encoding => STRING
+
+If supplied, sets an encoding to use to encode outgoing messages and decode
+incoming messages.
+
+=back
+
+=cut
+
 =head1 CONSTRUCTOR
 
 =cut
@@ -62,14 +129,14 @@ sub new
       %args,
 
       on_closed => sub {
-         my ( $self ) = @_;
+         my $self = shift;
 
          my $loop = $self->get_loop;
 
          $self->{pingtimer}->stop;
          $self->{pongtimer}->stop;
 
-         $on_closed->() if $on_closed;
+         $on_closed->( $self ) if $on_closed;
 
          undef $self->{connect_f};
          undef $self->{login_f};
@@ -117,58 +184,6 @@ sub encoder
    return $self->{encoder};
 }
 
-=head1 PARAMETERS
-
-The following named parameters may be passed to C<new> or C<configure>:
-
-=over 8
-
-=item on_message => CODE
-
-A CODE reference to the generic message handler; see C<MESSAGE HANDLING>
-below.
-
-=item on_message_* => CODE
-
-Any parameter whose name starts with C<on_message_> can be installed as a
-handler for a specific message, in preference to the generic handler. See
-C<MESSAGE HANDLING>.
-
-=item pingtime => NUM
-
-Amount of quiet time, in seconds, after a message is received from the peer,
-until a C<PING> will be sent to check it is still alive.
-
-=item pongtime => NUM
-
-Timeout, in seconds, after sending a C<PING> message, to wait for a C<PONG>
-response.
-
-=item on_ping_timeout => CODE
-
-A CODE reference to invoke if the peer fails to respond to a C<PING> message
-within the given timeout.
-
- $on_ping_timeout->( $irc )
-
-=item on_pong_reply => CODE
-
-A CODE reference to invoke when the peer successfully sends a C<PONG> in
-response of a C<PING> message.
-
- $on_pong_reply->( $irc, $lag )
-
-Where C<$lag> is the response time in (fractional) seconds.
-
-=item encoding => STRING
-
-If supplied, sets an encoding to use to encode outgoing messages and decode
-incoming messages.
-
-=back
-
-=cut
-
 sub configure
 {
    my $self = shift;
@@ -176,7 +191,7 @@ sub configure
 
    $self->{$_} = delete $args{$_} for grep m/^on_message/, keys %args;
 
-   for (qw( on_ping_timeout on_pong_reply )) {
+   for (qw( on_ping_timeout on_pong_reply on_irc_error )) {
       $self->{$_} = delete $args{$_} if exists $args{$_};
    }
 
@@ -260,8 +275,18 @@ sub on_read
 
    $pingtimer->is_running ? $pingtimer->reset : $pingtimer->start;
 
-   $self->Protocol::IRC::on_read( $$buffref );
-   return 0;
+   eval {
+      $self->Protocol::IRC::on_read( $$buffref );
+      1;
+   } and return 0;
+
+   my $e = "$@"; chomp $e;
+
+   $self->maybe_invoke_event( on_irc_error => $e )
+      and return 0;
+
+   $self->close_now;
+   die "$e\n";
 }
 
 =head2 $nick = $irc->nick
